@@ -15,13 +15,13 @@ from .constants import *
 from .logger.constants import *
 from .control.constants import *
 from .system.constants import *
+from .role_manager.constants import *
 from .unique_brain.constants import *
 from .packet import *
 
 from .util import now, log, logv
 from .pack import PackTuple, PackType, jdpack, jdunpack
 
-
 EV_CHANGE = "change"
 EV_DEVICE_CONNECT = "deviceConnect"
 EV_DEVICE_CHANGE = "deviceChange"
@@ -186,8 +186,6 @@ class Transport:
         # send a packet payload over the transport layer
         pass
 
-# TODO: replace randbytes
-
 
 def rand_u64():
     return bytearray([getrandbits(8) for _ in range(8)])
@@ -196,26 +194,35 @@ def rand_u64():
 class Bus(EventEmitter):
     """A Jacdac bus that managed devices, service client, registers."""
 
-    def __init__(self, transport: Transport, *,
+    def __init__(self, *,
+                 transports: List[Transport] = None,
                  device_id: str = None,
                  product_identifier: int = None,
                  firmware_version: str = None,
                  device_description: str = None,
                  disable_logger: bool = False,
+                 disable_role_manager: bool = False,
                  disable_brain: bool = False,
+                 disable_dev_tools: bool = False,
+                 hf2_portname: str = None,
                  default_logger_min_priority: int = JD_LOGGER_PRIORITY_SILENT,
-                 settings_file_name: str = None) -> None:
+                 role_manager_file_name: str = "./.jacdac/roles.json",
+                 settings_file_name: str = "./.jacdac/settings.json"
+                 ) -> None:
         """Instantiates a new Jacdac bus
 
         Args:
-            transport (Transport): packet transport
+            transports (List[Transport]): packet transports
             settings_file_name (str): Optional settings file location. Enables settings service.
             device_id (str, optional): Optional device identifier. Auto-generated if not specified.
             product_identifier (int, optional): Optional product identifier.
             device_description (str, optional): Optional device description.
             disable_logger (bool, optional): Disable the logger service. Defaults to False.
+            disable_role_manager (bool, optional): Disable the role manager service. Defaults to False.
             disable_brain (bool, optional): Disable unique brain service. Defaults to False.
             default_logger_min_priority (int, optional): Optional mininimum logger priority. Defaults to JD_LOGGER_PRIORITY_SILENT.
+            disable_dev_tools (bool, optional): Do not try to connect to developer tools server.
+            hf2_portname (str, optional): port name exposing HF2 packets.
         """
         super().__init__(self)
         self.devices: List['Device'] = []
@@ -223,6 +230,7 @@ class Bus(EventEmitter):
         self.all_clients: List['Client'] = []
         self.servers: List['Server'] = []
         self.logger: Optional[LoggerServer] = None
+        self.role_manager: Optional[RoleManagerServer] = None
         self.pipes: List['InPipe'] = []
 
         self.product_identifier = product_identifier
@@ -230,16 +238,25 @@ class Bus(EventEmitter):
         self.device_description = device_description
         self.disable_brain = disable_brain
         self.disable_logger = disable_logger
+        self.disable_role_manager = disable_role_manager
         self.default_logger_min_priority = default_logger_min_priority
         self.settings_file_name = settings_file_name
+        self.role_manager_file_name = role_manager_file_name
         self._event_counter = 0
 
         if device_id is None:
             device_id = rand_u64().hex()
         self.self_device = Device(self, device_id, bytearray(4))
         self.process_thread = threading.Thread(target=self._process_task)
-        self.transport = transport
-        self._sendq: queue.Queue[bytes] = queue.Queue()
+        self.transports: List[Transport] = transports or []
+        if not disable_dev_tools:
+            from .transports.ws import WebSocketTransport
+            self.transports.append(WebSocketTransport(DEVTOOLS_SOCKET_URL))
+        if hf2_portname:
+            from .transports.hf2 import HF2Transport
+            self.transports.append(HF2Transport(hf2_portname))
+
+        self._sendq: queue.Queue[Tuple[Transport, bytes]] = queue.Queue()
         self.pending_tasks: List[asyncio.Task[None]] = []
 
         self.loop = asyncio.new_event_loop()
@@ -266,8 +283,12 @@ class Bus(EventEmitter):
 
     def _sender(self):
         while True:
-            pkt = self._sendq.get()
-            self.transport.send(pkt)
+            c = self._sendq.get()
+            sender = c[0]
+            pkt = c[1]
+            for transport in self.transports:
+                if sender != transport:
+                    transport.send(pkt)
 
     def _process_task(self):
         loop = self.loop
@@ -279,6 +300,10 @@ class Bus(EventEmitter):
         if not self.disable_logger:
             self.logger = LoggerServer(self)
 
+        if not self.disable_role_manager:
+            self.role_manager = RoleManagerServer(
+                self, self.role_manager_file_name)
+
         if self.settings_file_name:
             from .settings.server import SettingsServer
             self.settings = SettingsServer(self)
@@ -301,9 +326,13 @@ class Bus(EventEmitter):
             loop.call_later(0.500, announce)
         loop.call_later(0.500, announce)
 
-        def process_later(pkt: bytes):
-            loop.call_soon_threadsafe(self.process_frame, pkt)
-        self.transport.on_receive = process_later
+        def process_later(sender: Transport, pkt: bytes):
+            loop.call_soon_threadsafe(self.process_frame, sender, pkt)
+
+        for transport in self.transports:
+            def process(pkt: bytes):
+                process_later(transport, pkt)
+            transport.on_receive = process
 
         try:
             loop.run_forever()
@@ -311,10 +340,13 @@ class Bus(EventEmitter):
             loop.run_until_complete(loop.shutdown_asyncgens())
             loop.close()
 
-    def process_frame(self, frame: bytes):
+    def process_frame(self, sender: Transport, frame: bytes):
         if frame[2] - frame[12] < 4:
             # single packet in frame
-            self.process_packet(JDPacket(frombytes=frame))
+            pkt = JDPacket(frombytes=frame, sender=sender)
+            self.process_packet(pkt)
+            # dispatch to other transports
+            self._queue_core(pkt)
         else:
             # split into frames
             ptr = 12
@@ -322,10 +354,12 @@ class Bus(EventEmitter):
                 sz = frame[ptr] + 4
                 pktbytes = frame[0:12] + frame[ptr:ptr+sz]
                 # log("PKT: {}-{} / {}", ptr, len(frame), pktbytes.hex())
-                pkt = JDPacket(frombytes=pktbytes)
+                pkt = JDPacket(frombytes=pktbytes, sender=sender)
                 if ptr > 12:
                     pkt.requires_ack = False
                 self.process_packet(pkt)
+                # dispatch to other transports
+                self._queue_core(pkt)
                 ptr += (sz + 3) & ~3
 
     def force_jd_thread(self):
@@ -365,14 +399,18 @@ class Bus(EventEmitter):
             self.emit(EV_DEVICE_CHANGE)
             self.emit(EV_CHANGE)
 
-    def _send_core(self, pkt: JDPacket):
+    def _queue_core(self, pkt: JDPacket):
         assert len(pkt._data) == pkt._header[12]
         pkt._header[2] = len(pkt._data) + 4
         buf = pkt._header + pkt._data
         crc = util.crc16(buf, 2)
+        sender = cast(Transport, pkt.sender)
         util.set_u16(buf, 0, crc)
         util.set_u16(pkt._header, 0, crc)
-        self._sendq.put(buf)
+        self._sendq.put((sender, buf))
+
+    def _send_core(self, pkt: JDPacket):
+        self._queue_core(pkt)
         self.process_packet(pkt)  # handle loop-back packet
 
     def clear_attach_cache(self):
@@ -595,7 +633,7 @@ class OutPipe(EventEmitter):
         self.port = cast(int, port)
         self.next_cnt = 0
 
-    @property
+    @ property
     def open(self):
         return not not self.port
 
@@ -882,7 +920,7 @@ class SensorServer(Server):
         self.streaming_interval = streaming_interval
         self._stream_task: Optional[Task[None]] = None
 
-    @abstractmethod
+    @ abstractmethod
     def send_reading(self):
         pass
 
@@ -1067,6 +1105,88 @@ class UniqueBrainServer(Server):
         super().__init__(bus, JD_SERVICE_CLASS_UNIQUE_BRAIN)
 
 
+class RoleManagerServer(Server):
+    def __init__(self, bus: Bus, file_name: str) -> None:
+        super().__init__(bus, JD_SERVICE_CLASS_ROLE_MANAGER)
+
+        from jacdac.settings_file import SettingsFile
+        self.settings = SettingsFile(file_name)
+        self.auto_bind = 1
+
+    def handle_packet(self, pkt: JDPacket):
+        self.auto_bind = self.handle_reg_u8(
+            pkt, JD_ROLE_MANAGER_REG_AUTO_BIND, self.auto_bind)
+
+        cmd = pkt.service_command
+
+        if cmd == JD_ROLE_MANAGER_CMD_LIST_REQUIRED_ROLES:
+            self.handle_list_required_roles(pkt)
+        elif cmd == JD_ROLE_MANAGER_CMD_LIST_STORED_ROLES:
+            self.handle_list_stored_roles(pkt)
+        elif cmd == JD_ROLE_MANAGER_CMD_CLEAR_ALL_ROLES:
+            self.handle_clear_all_roles(pkt)
+        elif cmd == JD_ROLE_MANAGER_CMD_GET_ROLE:
+            self.handle_get_role(pkt)
+        elif cmd == JD_ROLE_MANAGER_CMD_SET_ROLE:
+            self.handle_set_role(pkt)
+        elif cmd == JD_GET(JD_ROLE_MANAGER_REG_ALL_ROLES_ALLOCATED):
+            self.handle_all_roles_allocated(pkt)
+        else:
+            super().handle_packet(pkt)
+
+    def handle_list_required_roles(self, pkt: JDPacket):
+        pipe = OutPipe(self.bus, pkt)
+        for client in self.bus.all_clients:
+            device_id = bytearray(0)
+            service_class = client.service_class
+            service_index = client.service_index or 0
+            role = client.role
+            if client.device:
+                device_id = bytearray.fromhex(client.device.device_id)
+            payload = jdpack("b[8] u32 u8 s", device_id,
+                             service_class, service_index, role)
+            pipe.write(bytearray(payload))
+        pipe.close()
+
+    def handle_list_stored_roles(self, pkt: JDPacket):
+        pipe = OutPipe(self.bus, pkt)
+        for role in self.settings.list():
+            payload = self.settings.read(role)
+            pipe.write(payload)
+        pipe.close()
+
+    def handle_clear_all_roles(self, pkt: JDPacket):
+        self.settings.clear()
+        self.send_change_event()
+
+    def handle_get_role(self, pkt: JDPacket):
+        payload = pkt.unpack("b[8] u8")
+        device_id_b = cast(bytearray, payload[0])
+        device_id = device_id_b.hex()
+        service_index = cast(int, payload[1])
+
+        role = ""
+        for client in self.bus.all_clients:
+            if client.device and client.device.device_id == device_id and client.service_index == service_index:
+                role = client.role
+                break
+        self.send_report(JDPacket.packed(
+            JD_ROLE_MANAGER_CMD_GET_ROLE, "u[8] u8 s", device_id_b, service_index, role))
+
+    def handle_set_role(self, pkt: JDPacket):
+        payload = pkt.unpack("b[8] u8 s")
+        device_id_b = cast(bytearray, payload[0])
+        device_id = device_id_b.hex()
+        service_index = cast(int, payload[1])
+        role = cast(str, payload[2])
+        self.send_change_event()
+        # TODO
+
+    def handle_all_roles_allocated(self, pkt: JDPacket):
+        res = 0 if len(self.bus.unattached_clients) > 0 else 1
+        self.send_report(JDPacket.packed(pkt.service_command, "u8", res))
+
+
 class Client(EventEmitter):
     """Base class to define service clients."""
 
@@ -1077,7 +1197,6 @@ class Client(EventEmitter):
         self.pack_formats = pack_formats
         self.service_index = None
         self.device: Optional['Device'] = None
-        self.current_device:  Optional['Device'] = None
         self.role = role
         self._registers: List[RawRegisterClient] = []
         bus.unattached_clients.append(self)
@@ -1094,7 +1213,7 @@ class Client(EventEmitter):
                 return reg
         return None
 
-    @property
+    @ property
     def connected(self) -> bool:
         """Indicates if the client is a connected to a server"""
         return True if self.device else False
@@ -1127,10 +1246,10 @@ class Client(EventEmitter):
 
     def send_cmd(self, pkt: JDPacket):
         """Sends a command packet to the server"""
-        if self.current_device is None:
+        if self.device is None:
             return
         pkt.service_index = self.service_index
-        pkt.device_id = self.current_device.device_id
+        pkt.device_id = self.device.device_id
         pkt._header[3] |= JD_FRAME_FLAG_COMMAND
 
         self.bus.run(self.bus._send_core, pkt)
@@ -1157,6 +1276,9 @@ class Client(EventEmitter):
         self.debug("attached {}/{} to client {}", dev, service_idx, self.role)
         dev.clients.append(self)
         self.emit(EV_CONNECTED)
+        if self.bus.role_manager:
+            self.bus.role_manager.send_change_event()
+
         return True
 
     def _detach(self):
@@ -1170,6 +1292,8 @@ class Client(EventEmitter):
             self.bus.unattached_clients.append(self)
             self.bus.clear_attach_cache()
         self.emit(EV_DISCONNECTED)
+        if self.bus.role_manager:
+            self.bus.role_manager.send_change_event()
 
     def on_connect(self, handler: EventHandlerFn) -> UnsubscribeFn:
         """Registers an event handler when the client connects to a server
@@ -1228,16 +1352,16 @@ class SensorClient(Client):
         super().__init__(bus, service_class, pack_formats, role)
         self.preferred_interval = preferred_interval
 
-    @property
+    @ property
     def streaming_samples(self) -> Optional[int]:
         """Queries the current estimated streaming samples value"""
         return self.register(JD_REG_STREAMING_SAMPLES).value()
 
-    @property
+    @ property
     def streaming_interval(self) -> Optional[int]:
         return self.register(JD_REG_STREAMING_INTERVAL).value()
 
-    @property
+    @ property
     def streaming_preferred_interval(self) -> Optional[int]:
         return self.register(JD_REG_STREAMING_PREFERRED_INTERVAL).value()
 
@@ -1308,7 +1432,7 @@ class Device(EventEmitter):
         self._ctrl_client: Optional[Client] = None
         bus.devices.append(self)
 
-    @property
+    @ property
     def ctrl_client(self):
         if self._ctrl_client is None:
             self._ctrl_client = Client(
@@ -1316,23 +1440,23 @@ class Device(EventEmitter):
             self._ctrl_client._attach(self, 0)
         return self._ctrl_client
 
-    @property
+    @ property
     def announce_flags(self):
         return util.u16(self.services, 0)
 
-    @property
+    @ property
     def reset_count(self):
         return self.announce_flags & JD_CONTROL_ANNOUNCE_FLAGS_RESTART_COUNTER_STEADY
 
-    @property
+    @ property
     def packet_count(self):
         return self.services[2]
 
-    @property
+    @ property
     def is_connected(self):
         return self.clients != None
 
-    @property
+    @ property
     def short_id(self):
         return util.short_id(self.device_id)
 
@@ -1360,7 +1484,7 @@ class Device(EventEmitter):
         return True
         # TODO: return jacdac._rolemgr.getRole(self.deviceId, serviceIdx) == role
 
-    @property
+    @ property
     def num_service_classes(self):
         return len(self.services) >> 2
 
@@ -1415,5 +1539,5 @@ class Device(EventEmitter):
             if (c.broadcast and c.service_class == service_class) or \
                (not c.broadcast and c.service_index == pkt.service_index):
                 # log(`handle pkt at ${client.role} rep=${pkt.serviceCommand}`)
-                c.current_device = self
-                c.handle_packet_outer(pkt)
+                c.device = self
+                c.handle_packet_outer(pkt)
\ No newline at end of file
