@@ -1,7 +1,7 @@
 def const(v: int): return v
 # A shim for the micropython const keyword
 
-JD_VERSION = "0.0.19"
+JD_VERSION = "0.0.22"
 
 JD_SERIAL_HEADER_SIZE = const(16)
 JD_SERIAL_MAX_PAYLOAD_SIZE = const(236)
@@ -47,6 +47,8 @@ PIPE_COUNTER_MASK = const(0x001f)
 PIPE_CLOSE_MASK = const(0x0020)
 PIPE_METADATA_MASK = (0x0040)
 
+DEVTOOLS_URL = "http://localhost:8081"
+DEVTOOLS_SOCKET_URL = "ws://localhost:8081"
 
 def JD_GET(code: int):
     assert 0 <= code and code <= 0xfff
